#include <AFMotor.h> // Adafruit Motor Library
#define BTSerial Serial1 // Define BTSerial as Serial1 for Arduino Mega
#include <FastLED.h>

// LED configuration
#define NUM_LEDS_STRIP1 6 // Front LEDs
#define NUM_LEDS_STRIP2 6 // Rear LEDs
#define NUM_LEDS_STRIP3 6 // çakar

#define DATA_PIN_STRIP1 48
#define DATA_PIN_STRIP2 50
#define DATA_PIN_STRIP3 46
// LDR sensor pin
// #define LDR_PIN A0 // LDR for future improvements

// Create LED arrays
CRGB strip1[NUM_LEDS_STRIP1];
CRGB strip2[NUM_LEDS_STRIP2];
CRGB strip3[NUM_LEDS_STRIP3];

// Motor sürücü bağlantıları
AF_DCMotor motorFrontLeft(2);
AF_DCMotor motorFrontRight(1);
AF_DCMotor motorRearLeft(3);
AF_DCMotor motorRearRight(4);

// State variables
//bool headlightsOn = false; // Headlights based on LDR
bool turningLeft = false;
bool turningRight = false;
bool reversing = false;
bool braking = false;
bool moving = false;

void updateFrontLeds(){
    // Default: Turn all front LEDs off
    for (int i = 0; i < NUM_LEDS_STRIP1; i++) {
      strip1[i] = CRGB::Black;
    }

    // Turn on headlights if required
    if (moving) { // All LEDs Blue while moving
      for (int i = 0; i < NUM_LEDS_STRIP1; i++) {
        strip1[i] = CRGB::White;
      }
    } //else if (headlightsOn) { // Headlights logic for low light
      //for (int i = 0; i < NUM_LEDS_STRIP1; i++) {
      //  strip1[i] = CRGB::White;
      //}
    //}

    // Turn signal logic
    if (turningLeft) {
      strip1[0] = CRGB::Yellow;  // First LED
      strip1[1] = CRGB::Yellow;  // Second LED
    }

    // Right turn signal: Rightmost 2 LEDs blink yellow
    if (turningRight) {
      strip1[NUM_LEDS_STRIP1 - 1] = CRGB::Yellow; // Last LED
      strip1[NUM_LEDS_STRIP1 - 2] = CRGB::Yellow; // Second last LED
    }

    FastLED.show();
    }

    void updateRearLeds() {
      // Default: Turn all rear LEDs off
      for (int i = 0; i < NUM_LEDS_STRIP2; i++) {
        strip2[i] = CRGB::Black;
      }
      if (braking) {
        for (int i = 0; i < NUM_LEDS_STRIP2; i++) {
          strip2[i] = CRGB::Red;
        }
      }
      // Reverse lights
      if (reversing) {
        for (int i = 0; i < NUM_LEDS_STRIP2; i++) {
          strip2[i] = CRGB::Red;
        }
      }
      else if(moving) {
        for (int i = 0; i < NUM_LEDS_STRIP2; i++) {
          strip2[i] = CRGB::Black;
        } 
      }

    if (turningRight) {
      strip2[0] = CRGB::Yellow;  // First LED
      strip2[1] = CRGB::Yellow;  // Second LED
    }

    // Right turn signal: Rightmost 2 LEDs blink yellow
    if (turningLeft) {
      strip2[NUM_LEDS_STRIP2 - 1] = CRGB::Yellow; // Last LED
      strip2[NUM_LEDS_STRIP2 - 2] = CRGB::Yellow; // Second last LED
    }

      FastLED.show();
    }

void updatePoliceLeds() {
    static unsigned long lastUpdate = 0; // Track the time for blinking
    static bool toggle = false;          // Toggle state for red/blue alternation

    unsigned long currentTime = millis();

    // Update the lights every 500ms (adjust this value for faster or slower blinking)
    if (currentTime - lastUpdate > 500) {
        lastUpdate = currentTime;
        toggle = !toggle; // Switch between red and blue

        // Alternate colors for the police lights
        for (int i = 0; i < NUM_LEDS_STRIP3; i++) {
            if (toggle) {
                strip3[i] = (i % 2 == 0) ? CRGB::Red : CRGB::Blue; // Even LEDs Red, Odd LEDs Blue
            } else {
                strip3[i] = (i % 2 == 0) ? CRGB::Blue : CRGB::Red; // Even LEDs Blue, Odd LEDs Red
            }
        }
        FastLED.show();
    }
}

void setup() {
    // Initialize Bluetooth using BTSerial
    BTSerial.begin(9600);

    // Initialize LED strips
    FastLED.addLeds<WS2812B, DATA_PIN_STRIP1, GRB>(strip1, NUM_LEDS_STRIP1);
    FastLED.addLeds<WS2812B, DATA_PIN_STRIP2, GRB>(strip2, NUM_LEDS_STRIP2);
    FastLED.addLeds<WS2812B, DATA_PIN_STRIP3, GRB>(strip3, NUM_LEDS_STRIP3);
    // Initialize LDR sensor
    //pinMode(LDR_PIN, INPUT);

    // Motor hızını sıfırla
    motorFrontLeft.setSpeed(0);
    motorFrontRight.setSpeed(0);
    motorRearLeft.setSpeed(0);
    motorRearRight.setSpeed(0);

    // Motorları başlat
    motorFrontLeft.run(RELEASE);
    motorFrontRight.run(RELEASE);
    motorRearLeft.run(RELEASE);
    motorRearRight.run(RELEASE);
}

  void loop() {
    // Read the LDR sensor value
    //int lightLevel = analogRead(LDR_PIN); // Correctly read the analog LDR value
    //headlightsOn = (lightLevel < 500); // Adjust threshold as needed for low light detection

    // Update LED strips
    
  updateFrontLeds();
  updateRearLeds();
  updatePoliceLeds();
  if (BTSerial.available()) {
    char command = BTSerial.read();
    handleCommand(command);

    // Bluetooth komutlarına göre motorları kontrol et 
    if (command == 'W') { // ---------------- ÖNE HAREKET --------------------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(BACKWARD); // Sol ön motor
      motorFrontRight.run(BACKWARD); // Sağ ön motor
      motorRearLeft.run(FORWARD); // Arka sol motor
      motorRearRight.run(FORWARD); // Arka sağ motor
      moving = true;

    } else if (command == 'S') { // ----------- GERİ HAREKET ------------------
      motorFrontLeft.setSpeed(200);
      motorFrontRight.setSpeed(200);
      motorRearLeft.setSpeed(200);
      motorRearRight.setSpeed(200);

      motorFrontLeft.run(FORWARD);
      motorFrontRight.run(FORWARD);
      motorRearLeft.run(BACKWARD);
      motorRearRight.run(BACKWARD);
      moving = true;
      reversing = true;

    }  else if (command == 'A') { // ------------ sol -----------------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(FORWARD);
      motorFrontRight.run(BACKWARD);
      motorRearLeft.run(FORWARD);
      motorRearRight.run(BACKWARD);
      turningLeft = true;
      moving = true;
    } else if (command == 'D') { // ------------- sağ-----------------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(BACKWARD);
      motorFrontRight.run(FORWARD);
      motorRearLeft.run(BACKWARD);
      motorRearRight.run(FORWARD);
      turningRight = true;
      moving = true;
    } else if (command == 'Q') { // ------------- sol ön -----------------
      motorFrontLeft.setSpeed(0);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(0);

      motorFrontLeft.run(RELEASE);
      motorFrontRight.run(BACKWARD);
      motorRearLeft.run(FORWARD);
      motorRearRight.run(RELEASE);
      turningLeft = true;
      moving = true;
    } else if (command == 'E') { // ------------- sağ ön -----------------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(0);
      motorRearLeft.setSpeed(0);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(BACKWARD);
      motorFrontRight.run(RELEASE);
      motorRearLeft.run(RELEASE);
      motorRearRight.run(FORWARD);
      turningRight = true;
      moving = true;
    } else if (command == 'Z') { // ------------- sol arka -----------------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(0);
      motorRearLeft.setSpeed(0);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(FORWARD);
      motorFrontRight.run(RELEASE);
      motorRearLeft.run(RELEASE);
      motorRearRight.run(BACKWARD);
      turningLeft = true;
      reversing = true;
      moving = true;
    } else if (command == 'X') { // ------------- sağ arka-----------------
      motorFrontLeft.setSpeed(0);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(0);

      motorFrontLeft.run(RELEASE);
      motorFrontRight.run(FORWARD);
      motorRearLeft.run(BACKWARD);
      motorRearRight.run(RELEASE);
      turningRight = true;
      reversing = true;
      moving = true;
    } else if (command == 'G') { // ----------- SOLA DOĞRU ETRAFINDA DÖNME -------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(FORWARD);
      motorFrontRight.run(BACKWARD);
      motorRearLeft.run(BACKWARD);
      motorRearRight.run(FORWARD);
      turningLeft = true;
      moving = true;
    }else if (command == 'H') { // ---------- SAĞA DOĞRU ETRAFINDA DÖNME --------
      motorFrontLeft.setSpeed(150);
      motorFrontRight.setSpeed(150);
      motorRearLeft.setSpeed(150);
      motorRearRight.setSpeed(150);

      motorFrontLeft.run(BACKWARD);
      motorFrontRight.run(FORWARD);
      motorRearLeft.run(FORWARD);
      motorRearRight.run(BACKWARD);
      turningRight = true;
      moving = true;
    } else if (command == 'F') { // Stop
      motorFrontLeft.setSpeed(0);
      motorFrontRight.setSpeed(0);
      motorRearLeft.setSpeed(0);
      motorRearRight.setSpeed(0);

      motorFrontLeft.run(RELEASE);
      motorFrontRight.run(RELEASE);
      motorRearLeft.run(RELEASE);
      motorRearRight.run(RELEASE);
      
    braking = true;
    }
  }
}




void handleCommand(char command) {
  // Reset state variables
  turningLeft = false;
  turningRight = false;
  reversing = false;
  braking = false;
  moving = false;
  // Update LEDs after the command
  updateFrontLeds();
  updateRearLeds();
}
